#include <GL/glew.h>
#include <GL/gl.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <GL/glut.h>

inline int imax(int a, int b)
{
	return (a>b)?a:b;
}

char *textFileRead(const char *fn)
{
	FILE *fp;
	char *content = NULL;

	int count=0;

	if (fn != NULL)
	{
		fp = fopen(fn,"rt");

		if (fp != NULL)
		{
			fseek(fp, 0, SEEK_END);
			count = ftell(fp);
			rewind(fp);
			if (count > 0)
			{
				content = (char *) malloc(count + 1);
				count = fread(content, 1, count, fp);
				content[count] = '\0';
			}
			fclose(fp);
		}
	}
	return content;
}

char *dataFileRead(const char *fn)
{
	char *ret;
	int max_path_len=0;
	const char *paths[] = { 
		".",
		"..",
		"/usr/share/lightspark",
	};
	for(uint32_t i=0;i<sizeof(paths)/sizeof(const char*);i++)
		max_path_len=imax(strlen(paths[i]),max_path_len);
	max_path_len+=strlen(fn)+2;
	char* buf=new char[max_path_len];

	for (unsigned int i = 0; i < sizeof(paths)/sizeof(paths[0]); i++)
	{
		snprintf(buf, max_path_len, "%s/%s", paths[i], fn);
		ret = textFileRead(buf);
		if (ret)
		{
			delete[] buf;
			return ret;
		}
	}

	delete[] buf;
	return ret;
};

int main(int argc, char **argv){
	int gpu_program;
	glutInit(&argc, argv);
        glutCreateWindow("GLEW Test");
        glewExperimental = GL_TRUE;
	GLenum err = glewInit();
	if (GLEW_OK != err)
	{
		printf("%s\n", glewGetErrorString(err));;
		exit(1);
	}
	//Create render program
	assert(glCreateShader);
	GLuint f = glCreateShader(GL_FRAGMENT_SHADER);

	const char *fs = NULL;
	fs = dataFileRead("lightspark.frag");
	if(fs==NULL)
	{
		exit(1);
	}
	assert(glShaderSource);
	glShaderSource(f, 1, &fs,NULL);
	free((void*)fs);
	GLuint g = glCreateShader(GL_VERTEX_SHADER);

	bool ret=true;
	char str[1024];
	int a;
	assert(glCompileShader);
	glCompileShader(f);
	assert(glGetShaderInfoLog);
	glGetShaderInfoLog(f,1024,&a,str);

	fs = dataFileRead("lightspark.vert");
	if(fs==NULL)
	{
		exit(1);
	}
	glShaderSource(g, 1, &fs,NULL);
	free((void*)fs);

	glCompileShader(g);
	glGetShaderInfoLog(g,1024,&a,str);

	assert(glCreateProgram);
	gpu_program = glCreateProgram();
	assert(glAttachShader);
	glAttachShader(gpu_program,f);
	glAttachShader(gpu_program,g);

	assert(glLinkProgram);
	glLinkProgram(gpu_program);
	assert(glGetProgramiv);
	glGetProgramiv(gpu_program,GL_LINK_STATUS,&a);
	char info[1024];
	glGetProgramInfoLog(gpu_program,1024,NULL,info);
	if(a == GL_FALSE)
	{
		printf("program linking failed\n");
		printf("Reason : %s",info);
	}
	else
		printf("program linking succeeded\n");
}
